import 'dart:math';

import 'package:flutter/material.dart';
import 'package:photo_view/photo_view.dart';
import 'package:photo_view_example/screens/app_bar.dart';

class ControllerExample extends StatefulWidget {
  @override
  _ControllerExampleState createState() => _ControllerExampleState();
}

const double min = pi * -2;
const double max = pi * 2;

const double minScale = 0.03;
const double defScale = 0.1;
const double maxScale = 0.6;

class _ControllerExampleState extends State<ControllerExample> {
  PhotoViewControllerBase controller;
  PhotoViewScaleStateController scaleStateController;

  int calls = 0;

  @override
  void initState() {
    controller = PhotoViewController()
      ..scale = defScale
      ..outputStateStream.listen(onControllerState);

    scaleStateController = PhotoViewScaleStateController();
    super.initState();
  }

  void onControllerState(PhotoViewControllerValue value) {
    setState(() {
      calls += 1;
    });
  }

  @override
  void dispose() {
    controller.dispose();
    scaleStateController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
          height: MediaQuery.of(context).size.height,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.stretch,
            children: <Widget>[
              const ExampleAppBar(
                title: "Controller Examples",
                showGoBack: true,
              ),
              Flexible(
                  flex: 1,
                  child: ClipRect(
                    child: Stack(
                      children: <Widget>[
                        Positioned.fill(
                          child: PhotoView(
                            imageProvider:
                                const AssetImage("assets/large-image.jpg"),
                            controller: controller,
                            scaleStateController: scaleStateController,
                            enableRotation: true,
                            initialScale: defScale,
                            minScale: minScale,
                            maxScale: maxScale,
                          ),
                        ),
                        Positioned(
                          bottom: 0,
                          height: 220,
                          left: 0,
                          right: 0,
                          child: Container(
                              padding: const EdgeInsets.all(30.0),
                              child: StreamBuilder(
                                  stream: controller.outputStateStream,
                                  initialData: controller.value,
                                  builder: _streamBuild)),
                        )
                      ],
                    ),
                  )),
            ],
          )),
    );
  }

  Widget _streamBuild(BuildContext context, AsyncSnapshot snapshot) {
    if (snapshot.hasError || !snapshot.hasData) {
      return Container();
    }
    final PhotoViewControllerValue value = snapshot.data;
    return Column(
      children: <Widget>[
        Text(
          "Rotation ${value.rotation}",
          style: const TextStyle(color: Colors.white),
        ),
        SliderTheme(
            data: SliderTheme.of(context).copyWith(
                activeTrackColor: Colors.orange, thumbColor: Colors.orange),
            child: Slider(
                value: value.rotation.clamp(min, max),
                min: min,
                max: max,
                onChanged: (double newRotation) {
                  controller.rotation = newRotation;
                })),
        Text(
          "Scale ${value.scale}",
          style: const TextStyle(color: Colors.white),
        ),
        SliderTheme(
            data: SliderTheme.of(context).copyWith(
                activeTrackColor: Colors.orange, thumbColor: Colors.orange),
            child: Slider(
                value: value.scale.clamp(minScale, maxScale),
                min: minScale,
                max: maxScale,
                onChanged: (double newScale) {
                  controller.scale = newScale;
                })),
        Text(
          "ScaleState ${scaleStateController.scaleState}",
          style: const TextStyle(color: Colors.white),
        ),
      ],
    );
  }
}
